#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import shutil
import xml.etree.ElementTree as ET
import tkinter as tk
import tkinter.ttk as ttk
from basecontroller import BaseController, BaseView
import openpyxl
import settings
import assistant as ass


class ThisController(BaseController):

    def __init__(self, caller):
        super().__init__(caller)
        self.xlsx_filepath = None

    def create_new_dec(self):
        path = os.path.join(self.caller.path, 'app')
        from_ = os.path.join(path, 'template.xlsx')
        f = self._build_filename_from_datetime('.xlsx')
        filename = 'nova_declaracio_' + f
        to_ = os.path.join(self.caller.path, 'DECLARACIO_ACTUAL')
        self.to_ = os.path.join(to_, filename)
        shutil.copyfile(from_, self.to_)
        print("S'ha creat el nou arxiu " + self.to_)

    def create_xml(self, filepath, case):
        self.workbook = openpyxl.load_workbook(filepath, data_only=True)
        if case == 'BAL':
            self._export_bal()
        elif case == 'PIG':
            self._export_pig()
        elif case == 'EFE':
            self._export_efe()
        elif case == 'ECPN':
            self._export_ecpn()
        elif case == 'MEM':
            self._export_mem()

    def clear_sheet(self, filepath, case):
        self.workbook = openpyxl.load_workbook(filepath)
        self.xlsx_filepath = filepath
        if case == 'BAL':
            self._clear_bal()
        elif case == 'PIG':
            self._clear_pig()
        elif case == 'EFE':
            self._clear_efe()
        elif case == 'ECPN':
            self._clear_ecpn()
        elif case == 'MEM':
            self._clear_mem()

    def _clear_range(self, rangedict, description):
        for key in rangedict:
            sheet = self.workbook[key]
            for item in rangedict[key]:
                range_ = sheet[item]
                for row in range_:
                    for cell in row:
                        cell.value = None
        self.workbook.save(self.xlsx_filepath)
        txt = "S'han esborrat les dades " + description
        print()
        print(txt)

    def _clear_mem(self):
        self._clear_range(ass.RANGES_MEM, "de la Memòria")

    def _clear_pig(self):
        self._clear_range(ass.RANGES_PIG,
                          "del Compte de Pèrdues i Guanys")

    def _clear_bal(self):
        self._clear_range(ass.RANGES_BAL, "del Balanç")

    def _clear_efe(self):
        self._clear_range(ass.RANGES_EFE,
                          "de l'Estat de Fluxos d'Efectiu")

    def _clear_ecpn(self):
        self._clear_range(ass.RANGES_ECPN,
                          "de l'Estat de Canvis de Patrimoni Net")

    def _export_bal(self):
        d = self._create_matrix('BAL', ass.BAL)
        response = self._send_socket(command=5, data=[('BAL', d)])
        self._manage_xml_response(response, 'bal')

    def _export_pig(self):
        d = self._create_matrix('PIG', ass.PIG)
        response = self._send_socket(command=5, data=[('PIG', d)])
        self._manage_xml_response(response, 'pig')

    def _export_mem(self):
        items = []
        data = self._create_matrix('N3', ass.N3)
        items.append(('N3', data))
        data = self._create_matrix('N5', ass.N5)
        items.append(('N5', data))
        data = self._create_matrix('N6', ass.N6)
        items.append(('N6', data))
        data = self._create_matrix('N7', ass.N7)
        items.append(('N7', data))
        data = self._create_matrix('N8', ass.N8)
        items.append(('N8', data))
        data = self._create_matrix_offset('N8', ass.N8_offset)
        items.append(('N8_offset', data))
        data = self._create_matrix('N9', ass.N9)
        items.append(('N9', data))
        data = self._create_matrix_offset('N10', ass.N10_1_offset)
        items.append(('N10_1_offset', data))
        data = self._create_matrix_offset('N10', ass.N10_2_offset)
        items.append(('N10_2_offset', data))
        data = self._create_matrix('N10', ass.N10)
        items.append(('N10', data))
        data = self._create_matrix_offset('N11-12', ass.N11_1_offset)
        items.append(('N11_1_offset', data))
        data = self._create_matrix_offset('N11-12', ass.N11_2_offset)
        items.append(('N11_2_offset', data))
        data = self._create_matrix_offset('N11-12', ass.N11_3_offset)
        items.append(('N11_3_offset', data))
        data = self._create_matrix_offset('N11-12', ass.N11_4_offset)
        items.append(('N11_4_offset', data))

        response = self._send_socket(command=5, data=items)
        self._manage_xml_response(response, 'mem')

    def _create_matrix(self, sheet, matrix):
        sheet = self.workbook[sheet]
        data = {}
        for row in matrix:
            r = str(row)
            data[r] = []
            for col in matrix[row]:
                val = self._get_cell_value(sheet, row, col)
                data[r].append(val)
        return data

    def _create_matrix_offset(self, sheet, matrix):
        sheet = self.workbook[sheet]
        data = []
        row = matrix['row']
        limit = row + matrix['length']
        while row < limit:
            line = []
            line_data = False
            for col in matrix['cols']:
                val = self._get_cell_value(sheet, row, col)
                line.append(val)
                if val and not line_data:
                    line_data = True
            if line_data:
                data.append(line)
            row += 1
        return data

    def _get_cell_value(self, sheet, row, col):
        if type(col) is tuple:
            cell = sheet.cell(row=row, column=col[0] + 1)
            if col[1] == 'DATE':
                if cell.value:
                    val = self._get_date_as_string(cell.value)
                else:
                    val = '-'
            if col[1] == 'STRING':
                if cell.value:
                    val = str(cell.value)
                else:
                    val = '-'
        else:
            cell = sheet.cell(row=row, column=col + 1)
            if cell.value:
                if cell.value == int(cell.value):
                    val = int(cell.value)
                else:
                    val = cell.value
            else:
                return 0
        return val

    def _get_date_as_string(self, value):
        return value.strftime("%d/%m/%Y")

    def _manage_xml_response(self, response, filename):
        if not response:
            txt = "Error 101 - No s'ha pogut connectar amb el socket server"
            print(txt)
        else:
            if response[0] in (0, 1, 2, 4):
                txt = response[1]
                print(txt)
            elif response[0] == 3:
                text = response[1]
                root = ET.fromstring(text)
                tree = ET.ElementTree(root)
                path = os.path.join(self.path, 'DECLARACIO_ACTUAL')
                f = self._build_filename_from_datetime('.xml')
                filepath = os.path.join(path, filename + '_' + f)
                try:
                    tree.write(filepath, encoding="utf-8",
                               xml_declaration=True)
                except:
                    tree.write(filepath)
                txt = "S'ha generat l'arxiu " + filepath
                print(txt)


class MainView(BaseView):

    def __init__(self, path):
        super().__init__(path)
        self.controller = ThisController(caller=self)
        self._draw()

    def _draw_app_buttons(self):
        f2 = tk.Frame(self.top)
        f21 = tk.Frame(f2, highlightbackground="black",
                       highlightthickness=0.5)
        f211 = tk.Frame(f21)
        lbl1 = ttk.Label(f211, text='Seleccioni document  ')
        lbl1.pack(side=tk.LEFT)
        self.cbo1 = ttk.Combobox(f211,
                                 values=settings.XLSX_SHEETS,
                                 widt=10)
        self.cbo1.set('BAL')
        self.cbo1.pack(side=tk.RIGHT)
        f211.pack(padx=2, pady=5)
        f212 = tk.Frame(f21)
        self.b4 = ttk.Button(f212, text="Crear fitxer XML",
                             command=self._create_xml)

        self.b4.pack(side=tk.LEFT, pady=5, padx=5)
        self.b5 = ttk.Button(f212, text="Esborrar dades",
                             command=self._clear_sheet)
        self.b5.pack(side=tk.RIGHT, pady=5, padx=5)

        f212.pack(pady=0, fill=tk.BOTH)
        f21.pack(side=tk.LEFT)
        f22 = tk.Frame(f2)
        self.b6 = ttk.Button(f22, text="Crear nova declaració",
                             command=self._new_dec)
        self.b6.pack(side=tk.TOP)

        f22.pack(side=tk.RIGHT, padx=10, fill=tk.BOTH)
        f2.pack(pady=5, padx=5, fill=tk.BOTH)

    def _create_xml(self):
        self.text.clear()
        case = self.cbo1.get()
        if case not in settings.XLSX_SHEETS:
            print('Error 101 - Sel·leccioni un document')
        else:
            dir_ = os.path.join(self.path, 'DECLARACIO_ACTUAL')
            filepath = self._select_open_file(self,
                                              initialdir=dir_,
                                              filetypes=[('arxius XLSX',
                                                          '*.xlsx')])
            if not filepath:
                return
            else:
                self.controller.create_xml(filepath, case)

    def _clear_sheet(self):
        case = self.cbo1.get()
        if case not in settings.XLSX_SHEETS:
            print('Error 101 - Sel·leccioni un document')
        else:
            dir_ = os.path.join(self.path, 'DECLARACIO_ACTUAL')
            filepath = self._select_open_file(self,
                                              initialdir=dir_,
                                              filetypes=[('arxius XLSX',
                                                          '*.xlsx')])
            if not filepath:
                return
            else:
                self.controller.clear_sheet(filepath, case)

    def _new_dec(self):
        self.text.clear()
        self.controller.create_new_dec()

    def set_status(self, value):
        self._set_status(value)
        if value == settings.STATUS_INITIAL:
            self.b4.state(["disabled"])
            self.b5.state(["disabled"])
            self.b6.state(["disabled"])
            # self.cbo1.state(["disabled"])
        elif value == settings.STATUS_ONLY_EXIT:
            self.b4.state(["disabled"])
            self.b5.state(["disabled"])
            self.b6.state(["disabled"])
            # self.cbo1.state(["disabled"])
        elif value == settings.STATUS_ACTIVE:
            self.b4.state(["!disabled"])
            self.b5.state(["!disabled"])
            self.b6.state(["!disabled"])
            # self.cbo1.state(["readonly"])
